(* Note added due to LGPL terms.

This file was edited by Valerio Bigiani, AKA The Bigg, starting from
6 November 2005. All changes for this file are listed in
diffs/src.arch_cygwin.ml.diff file, as the output of a diff -p -E -b command.

It was originally taken from Westley Weimer's WeiDU 185. *)

(* CygWin Arch-Specific Definitions *)

(* Query the Windows Registry to find out about Infinity Engine games. *)
let unixifydir str = Str.global_replace (Str.regexp (Str.quote "\\")) "/" str

external registry_path : unit -> string = "get_bg2main_path" 
external bg_registry_path : unit -> string = "get_bgmain_path" 
external iwd_registry_path : unit -> string = "get_iwdmain_path" 
external iwd2_registry_path : unit -> string = "get_iwd2main_path" 
external pst_registry_path : unit -> string = "get_pstmain_path" 

let registry_paths = ref [ 
  unixifydir (registry_path ()) ;
  unixifydir (bg_registry_path ()) ;
  unixifydir (iwd_registry_path ()) ;
  unixifydir (iwd2_registry_path ()) ;
  unixifydir (pst_registry_path ()) ; 
  "C:/Program Files/Black Isle/BGII - SoA/" 
] 

(* no slash removal: helps make shell scripts work better *)
let slash_to_backslash s = s 

let backslash_to_slash s =
  let s = Str.global_replace (Str.regexp "\\\\") "/" s in
  s

(* how to view a text file *)
let view_command = "less" 

let do_auto_update = false 

(* handle AT_EXIT ~VIEW mydir\myfile.txt~ *)
let handle_view_command s skip =
  let result = ref s in
  List.iter (fun view_regexp ->
    if Str.string_match view_regexp !result 0 then begin
      result := Str.replace_first view_regexp view_command !result ;
      result := Str.global_replace (Str.regexp (Str.quote "\\")) "/" !result;
    end) [Str.regexp_case_fold "^VIEW" ; Str.regexp_case_fold "^NOTEPAD" ;
          Str.regexp_case_fold "^EXPLORER"];
  if skip && (s <> !result) then result := "";
  let s = !result in
  s

let glob str fn = failwith "no globbing support"

let create_process_env = Unix.create_process_env

let biff_path_separator = "\\\\"

let cd_regexp = Str.regexp "^[CH]D[0-9]+.*=\\([^\r\n]*\\)" 
